@@ -24,8 +24,6 @@ namespace GLib {
 
 	using System;
 	using System.Collections;
-	using System.Collections.Generic;
-	using System.IO;
 	using System.Reflection;
 	using System.Runtime.InteropServices;
 
@@ -39,11 +37,6 @@ namespace GLib {
 			this.val = val;
 		}
 
-		public static GType FromName (string native_name)
-		{
-			return new GType (g_type_from_name (native_name));
-		}
-		
 		public static readonly GType Invalid = new GType ((IntPtr) TypeFundamentals.TypeInvalid);
 		public static readonly GType None = new GType ((IntPtr) TypeFundamentals.TypeNone);
 		public static readonly GType Interface = new GType ((IntPtr) TypeFundamentals.TypeInterface);
@@ -52,8 +45,6 @@ namespace GLib {
 		public static readonly GType Boolean = new GType ((IntPtr) TypeFundamentals.TypeBoolean);
 		public static readonly GType Int = new GType ((IntPtr) TypeFundamentals.TypeInt);
 		public static readonly GType UInt = new GType ((IntPtr) TypeFundamentals.TypeUInt);
-		public static readonly GType Long = new GType ((IntPtr) TypeFundamentals.TypeLong);
-		public static readonly GType ULong = new GType ((IntPtr) TypeFundamentals.TypeULong);
 		public static readonly GType Int64 = new GType ((IntPtr) TypeFundamentals.TypeInt64);
 		public static readonly GType UInt64 = new GType ((IntPtr) TypeFundamentals.TypeUInt64);
 		public static readonly GType Enum = new GType ((IntPtr) TypeFundamentals.TypeEnum);
@@ -63,11 +54,10 @@ namespace GLib {
 		public static readonly GType String = new GType ((IntPtr) TypeFundamentals.TypeString);
 		public static readonly GType Pointer = new GType ((IntPtr) TypeFundamentals.TypePointer);
 		public static readonly GType Boxed = new GType ((IntPtr) TypeFundamentals.TypeBoxed);
-		public static readonly GType Param = new GType ((IntPtr) TypeFundamentals.TypeParam);
 		public static readonly GType Object = new GType ((IntPtr) TypeFundamentals.TypeObject);
 
-		static Dictionary<IntPtr, Type> types = new Dictionary<IntPtr, Type> ();
-		static Dictionary<Type, GType> gtypes = new Dictionary<Type, GType> ();
+		static Hashtable types = new Hashtable ();
+		static Hashtable gtypes = new Hashtable ();
 
 		public static void Register (GType native_type, System.Type type)
 		{
@@ -77,14 +67,11 @@ namespace GLib {
 				gtypes[type] = native_type;
 		}
 
-		[DllImport("libgobject-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libgobject-2.0-0.dll")]
 		static extern void g_type_init ();
 
 		static GType ()
 		{
-			if (!GLib.Thread.Supported)
-				GLib.Thread.Init ();
-
 			g_type_init ();
 
 			Register (GType.Char, typeof (sbyte));
@@ -97,7 +84,6 @@ namespace GLib {
 			Register (GType.Float, typeof (float));
 			Register (GType.Double, typeof (double));
 			Register (GType.String, typeof (string));
-			Register (GType.Pointer, typeof (IntPtr));
 			Register (GType.Object, typeof (GLib.Object));
 
 			// One-way mapping
@@ -108,39 +94,26 @@ namespace GLib {
 		{
 			GType gtype;
 
-			if (gtypes.TryGetValue (type, out gtype))
-				return gtype;
-			
+			if (gtypes.Contains (type))
+				return (GType)gtypes[type];
+
 			if (type.IsSubclassOf (typeof (GLib.Object))) {
 				gtype = GLib.Object.LookupGType (type);
 				Register (gtype, type);
 				return gtype;
 			}
 
-			if (type.IsEnum) {
-				GTypeTypeAttribute geattr;
-				GTypeAttribute gattr;
-				if ((geattr = (GTypeTypeAttribute)Attribute.GetCustomAttribute (type, typeof (GTypeTypeAttribute), false)) != null) {
-					gtype = geattr.Type;
-				} else if ((gattr = (GTypeAttribute)Attribute.GetCustomAttribute (type, typeof (GTypeAttribute), false)) != null) {
-					// This should never happen for generated code, keep it in place for other users of the API.
-					var pi = gattr.WrapperType.GetProperty ("GType", BindingFlags.Public | BindingFlags.Static);
-					gtype = (GType)pi.GetValue (null, null);
-				} else
-					gtype = ManagedValue.GType;
-			} else {
-				GTypeTypeAttribute geattr;
-				PropertyInfo pi;
-				if ((geattr = (GTypeTypeAttribute)Attribute.GetCustomAttribute (type, typeof (GTypeTypeAttribute), false)) != null) {
-					gtype = geattr.Type;
-				} else if ((pi = type.GetProperty ("GType", BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static | BindingFlags.FlattenHierarchy)) != null) {
-					gtype = (GType)pi.GetValue (null, null);
-				} else if (type.IsSubclassOf (typeof (GLib.Opaque)))
-					gtype = GType.Pointer;
-				else
-					gtype = ManagedValue.GType;
-			}
-
+			PropertyInfo pi = type.GetProperty ("GType", BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static | BindingFlags.FlattenHierarchy);
+			if (pi != null)
+				gtype = (GType) pi.GetValue (null, null);
+			else if (type.IsDefined (typeof (GTypeAttribute), false)) {
+				GTypeAttribute gattr = (GTypeAttribute)Attribute.GetCustomAttribute (type, typeof (GTypeAttribute), false);
+				pi = gattr.WrapperType.GetProperty ("GType", BindingFlags.Public | BindingFlags.Static);
+				gtype = (GType) pi.GetValue (null, null);
+			} else if (type.IsSubclassOf (typeof (GLib.Opaque)))
+				gtype = GType.Pointer;
+			else
+				gtype = ManagedValue.GType;
 
 			Register (gtype, type);
 			return gtype;
@@ -172,51 +145,18 @@ namespace GLib {
 
 		public static Type LookupType (IntPtr typeid)
 		{
-			Type result;
-			if (types.TryGetValue (typeid, out result))
-				return result;
+			if (types.Contains (typeid))
+				return (Type)types[typeid];
 
 			string native_name = Marshaller.Utf8PtrToString (g_type_name (typeid));
 			string type_name = GetQualifiedName (native_name);
-			Assembly[] assemblies = (Assembly[]) AppDomain.CurrentDomain.GetAssemblies ().Clone ();
-			foreach (Assembly asm in assemblies) {
+			Type result = null;
+			foreach (Assembly asm in AppDomain.CurrentDomain.GetAssemblies ()) {
 				result = asm.GetType (type_name);
 				if (result != null)
 					break;
 			}
 
-			if (result == null) {
-				// Because of lazy loading of references, it's possible the type's assembly
-				// needs to be loaded.  We will look for it by name in the references of
-				// the currently loaded assemblies.  Hopefully a recursive traversal is
-				// not needed. We avoid one for now because of problems experienced
-				// in a patch from bug #400595, and a desire to keep memory usage low
-				// by avoiding a complete loading of all dependent assemblies.
-				string ns = type_name.Substring (0, type_name.LastIndexOf ('.'));
-				string asm_name = ns.ToLower ().Replace ('.', '-') + "-sharp";
-				foreach (Assembly asm in assemblies) {
-					foreach (AssemblyName ref_name in asm.GetReferencedAssemblies ()) {
-						if (ref_name.Name != asm_name)
-							continue;
-						try {
-							string asm_dir = Path.GetDirectoryName (asm.Location);
-							Assembly ref_asm;
-							if (File.Exists (Path.Combine (asm_dir, ref_name.Name + ".dll")))
-								ref_asm = Assembly.LoadFrom (Path.Combine (asm_dir, ref_name.Name + ".dll"));
-							else
-								ref_asm = Assembly.Load (ref_name);
-							result = ref_asm.GetType (type_name);
-							if (result != null)
-								break;
-						} catch (Exception) {
-							/* Failure to load a referenced assembly is not an error */
-						}
-					}
-					if (result != null)
-						break;
-				}
-			}
-
 			Register (new GType (typeid), result);
 			return result;
 		}
@@ -250,11 +190,8 @@ namespace GLib {
 			return val.GetHashCode ();
 		}
 
-		[DllImport("libgobject-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libgobject-2.0-0.dll")]
 		static extern IntPtr g_type_name (IntPtr raw);
-		
-		[DllImport("libgobject-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
-		static extern IntPtr g_type_from_name (string name);
 
 		public override string ToString ()
 		{
