@@ -22,13 +22,16 @@
 namespace GLib {
 
 	using System;
+	using System.Collections;
 	using System.Runtime.InteropServices;
 
 	internal class ToggleRef {
 
+		bool hardened;
 		IntPtr handle;
 		object reference;
 		GCHandle gch;
+		Hashtable signals;
 
 		public ToggleRef (GLib.Object target)
 		{
@@ -44,11 +47,26 @@ namespace GLib {
 				if (reference is WeakReference) {
 					WeakReference weak = reference as WeakReference;
 					return weak.IsAlive;
-				}
+				} else if (reference == null)
+					return false;
 				return true;
 			}
 		}
 
+		public IntPtr Handle {
+			get {
+				return handle;
+			}
+		}
+
+		public Hashtable Signals {
+			get {
+				if (signals == null)
+					signals = new Hashtable ();
+				return signals;
+			}
+		}
+
 		public GLib.Object Target {
 			get {
 				if (reference is GLib.Object)
@@ -61,11 +79,33 @@ namespace GLib {
 
 		public void Free ()
 		{
-			g_object_remove_toggle_ref (handle, ToggleNotifyCallback, (IntPtr) gch);
+			foreach (Signal s in Signals.Values)
+				s.Free ();
+			Signals.Clear ();
+			if (hardened)
+				g_object_unref (handle);
+			else
+				g_object_remove_toggle_ref (handle, ToggleNotifyCallback, (IntPtr) gch);
 			reference = null;
 			gch.Free ();
 		}
 
+		internal void Harden ()
+		{
+			// Added for the benefit of GnomeProgram.  It releases a final ref in
+			// an atexit handler which causes toggle ref notifications to occur after 
+			// our delegates are gone, so we need a mechanism to override the 
+			// notifications.  This method effectively leaks all objects which invoke it, 
+			// but since it is only used by Gnome.Program, which is a singleton object 
+			// with program duration persistence, who cares.
+
+			g_object_ref (handle);
+			g_object_remove_toggle_ref (handle, ToggleNotifyCallback, (IntPtr) gch);
+			if (reference is WeakReference)
+				reference = (reference as WeakReference).Target;
+			hardened = true;
+		}
+
 		void Toggle (bool is_last_ref)
 		{
 			if (is_last_ref && reference is GLib.Object)
@@ -106,6 +146,9 @@ namespace GLib {
 		[DllImport("libgobject-2.0-0.dll")]
 		static extern void g_object_remove_toggle_ref (IntPtr raw, ToggleNotifyHandler notify_cb, IntPtr data);
 
+		[DllImport("libgobject-2.0-0.dll")]
+		static extern IntPtr g_object_ref (IntPtr raw);
+
 		[DllImport("libgobject-2.0-0.dll")]
 		static extern void g_object_unref (IntPtr raw);
 
